<?php

namespace Drupal\lb_block_decorator\Form;

use Drupal\Core\Ajax\AjaxFormHelperTrait;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\lb_block_decorator\BlockDecoratorPluginManager;
use Drupal\layout_builder\Context\LayoutBuilderContextTrait;
use Drupal\layout_builder\Controller\LayoutRebuildTrait;
use Drupal\layout_builder\LayoutBuilderHighlightTrait;
use Drupal\layout_builder\LayoutTempstoreRepositoryInterface;
use Drupal\layout_builder\SectionComponentTrait;
use Drupal\layout_builder\SectionStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for wrapper markup to a block.
 *
 * @internal
 *   Form classes are internal.
 */
class BlockDecoratorForm extends FormBase implements BlockDecoratorFormInterface {

  use AjaxFormHelperTrait;
  use LayoutBuilderContextTrait;
  use LayoutBuilderHighlightTrait;
  use LayoutRebuildTrait;
  use SectionComponentTrait;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The section storage.
   *
   * @var \Drupal\layout_builder\SectionStorageInterface
   */
  protected $sectionStorage;

  /**
   * The layout section delta.
   *
   * @var int
   */
  protected $delta;

  /**
   * The uuid of the block component.
   *
   * @var string
   */
  protected $uuid;

  /**
   * The layout tempstore repository.
   *
   * @var \Drupal\layout_builder\LayoutTempstoreRepositoryInterface
   */
  protected $layoutTempstoreRepository;

  /**
   * The BlockDecorator Plugin Manager.
   *
   * @var \Drupal\lb_block_decorator\BlockDecoratorPluginManager
   */
  protected $blockDecoratorManager;

  /**
   * Constructs a BlockWrapperForm object.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   * @param \Drupal\layout_builder\LayoutTempstoreRepositoryInterface $layout_tempstore_repository
   *   The layout tempstore repository.
   * @param \Drupal\lb_block_decorator\BlockDecoratorPluginManager $block_decorator_manager
   *   The block decorator plugin manager.
   */
  public function __construct(FormBuilderInterface $form_builder, LayoutTempstoreRepositoryInterface $layout_tempstore_repository, BlockDecoratorPluginManager $block_decorator_manager) {
    $this->formBuilder = $form_builder;
    $this->layoutTempstoreRepository = $layout_tempstore_repository;
    $this->blockDecoratorManager = $block_decorator_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder'),
      $container->get('layout_builder.tempstore_repository'),
      $container->get('plugin.manager.block_decorator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lb_block_decorator_block_decorator';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, SectionStorageInterface $section_storage = NULL, $delta = NULL, $uuid = NULL) {
    $form_state->set('workspace_safe', TRUE);
    $this->sectionStorage = $section_storage;
    $this->delta = $delta;
    $this->uuid = $uuid;
    $component = $this->getCurrentComponent();

    $decorator_id = $form_state->getUserInput()['plugin_container']['decorator_id'] ?? $component->get('decorator_id');
    $decorators = $this->blockDecoratorManager->getFilteredDefinitions('lb_block_decorator');

    foreach ($decorators as $decorator) {
      $available_decorators[$decorator['id']] = $decorator['label'];
    }

    $form['plugin_container'] = [
      '#type' => 'container',
      '#prefix' => '<div id="decorator-plugin-container">',
      '#suffix' => '</div>',
      '#group' => 'configuration_container',
      '#parents' => ['plugin_container'],
      '#tree' => TRUE,
      'decorator_id' => [
        '#title' => $this->t('Plugin'),
        '#type' => 'select',
        '#empty_option' => $this->t('- Select -'),
        '#empty_value' => '',
        '#options' => $available_decorators,
        '#required' => TRUE,
        '#default_value' => $decorator_id,
        '#ajax' => [
          'callback' => [$this, 'loadPluginForm'],
          'wrapper' => 'decorator-plugin-container',
          'effect' => 'fade',
        ],
      ],
    ];
    if (!$form_state->getValue('plugin_container')) {
      // Give plugins a place to stash their values on initial load.
      $form_state->setValue('plugin_container', []);
    }

    if ($decorator_id) {
      /** @var \Drupal\lb_block_decorator\BlockDecoratorInterface $decorator_plugin */
      $decorator_plugin = $this->blockDecoratorManager->createInstance($decorator_id);
      $form_state->set('decorator_plugin', $decorator_plugin);
      $form['#parents'] = [];
      $subform_state = SubformState::createForSubform($form['plugin_container'], $form, $form_state);
      $component = $this->getCurrentComponent();
      if (!empty($component->get('decorator_configuration'))) {
        $decorator_plugin->setConfiguration($component->get('decorator_configuration'));
      }
      $form['plugin_container'] += $decorator_plugin->buildConfigurationForm([], $subform_state);

      $form['plugin_container']['actions'] = [
        '#weight' => 40,
        'submit' => [
          '#type' => 'submit',
          '#value' => $this->t('Save'),
        ],
        'remove' => [
          '#type' => 'submit',
          '#value' => $this->t('Remove'),
          '#name' => 'remove',
        ],
      ];
    }

    $form['#attributes']['data-layout-builder-target-highlight-id'] = $this->blockUpdateHighlightId($this->uuid);

    if ($this->isAjax()) {
      $form['plugin_container']['actions']['submit']['#ajax']['callback'] = '::ajaxSubmit';
      $form['plugin_container']['actions']['remove']['#ajax']['callback'] = '::ajaxSubmit';
    }

    return $form;
  }

  /**
   * AJAX callback to load the plugin form.
   */
  public function loadPluginForm(array $form, FormStateInterface $form_state): array {
    return $form['plugin_container'];
  }

  /**
   * Closes the tray after submit handler succeeded.
   */
  protected function successfulAjaxSubmit(array $form, FormStateInterface $form_state) {
    return $this->rebuildAndClose($this->sectionStorage);
  }

  /**
   * @{inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $component = $this->getCurrentComponent();
    if ($form_state->getTriggeringElement()['#name'] == 'remove') {
      $component->set('decorator_id', NULL);
      $component->set('decorator_configuration', []);
    }
    elseif ($decorator_plugin = $form_state->get('decorator_plugin')) {
      $subform_state = SubformState::createForSubform($form['plugin_container'], $form, $form_state);
      $subform_state->set('section_component', $component);
      $decorator_plugin->submitConfigurationForm($form, $subform_state);
    }
    $this->updateTempstore($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function updateTempstore(FormStateInterface $form_state): void {
    $this->layoutTempstoreRepository->set($this->sectionStorage);
    $form_state->setRedirectUrl($this->sectionStorage->getLayoutBuilderUrl());
  }

  /**
   * Provides a title callback.
   *
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   * @param int $delta
   *   The original delta of the section.
   * @param string $uuid
   *   The UUID of the block being updated.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The title for the block visibility form.
   */
  public function title(SectionStorageInterface $section_storage, $delta, $uuid) {
    $block_label = $section_storage
      ->getSection($delta)
      ->getComponent($uuid)
      ->getPlugin()
      ->label();

    return $this->t('Configure a decorator for the @block_label block', ['@block_label' => $block_label]);
  }

}
